@@ -1,5 +1,6 @@
+import copy
 from datetime import datetime
-from typing import NamedTuple, Callable
+from typing import NamedTuple, Callable, TYPE_CHECKING
 
 from kivy.app import App
 from kivy.factory import Factory
@@ -15,6 +16,12 @@ from electrum.gui.kivy.i18n import _
 
 from electrum.util import InvalidPassword
 from electrum.address_synchronizer import TX_HEIGHT_LOCAL
+from electrum.wallet import CannotBumpFee, CannotDoubleSpendTx
+from electrum.transaction import Transaction, PartialTransaction
+from ...util import address_colors
+
+if TYPE_CHECKING:
+    from ...main_window import ElectrumWindow
 
 
 Builder.load_string('''
@@ -27,6 +34,7 @@ Builder.load_string('''
     can_broadcast: False
     can_rbf: False
     fee_str: ''
+    feerate_str: ''
     date_str: ''
     date_label:''
     amount_str: ''
@@ -65,6 +73,9 @@ Builder.load_string('''
                     BoxLabel:
                         text: _('Transaction fee') if root.fee_str else ''
                         value: root.fee_str
+                    BoxLabel:
+                        text: _('Transaction fee rate') if root.feerate_str else ''
+                        value: root.feerate_str
                 TopLabel:
                     text: _('Transaction ID') + ':' if root.tx_hash else ''
                 TxHashLabel:
@@ -93,6 +104,11 @@ Builder.load_string('''
                 height: '48dp'
                 icon: 'atlas://electrum/gui/kivy/theming/light/qrcode'
                 on_release: root.show_qr()
+            Button:
+                size_hint: 0.5, None
+                height: '48dp'
+                text: _('Label')
+                on_release: root.label_dialog()
             Button:
                 size_hint: 0.5, None
                 height: '48dp'
@@ -111,11 +127,17 @@ class TxDialog(Factory.Popup):
 
     def __init__(self, app, tx):
         Factory.Popup.__init__(self)
-        self.app = app
+        self.app = app  # type: ElectrumWindow
         self.wallet = self.app.wallet
-        self.tx = tx
+        self.tx = tx  # type: Transaction
         self._action_button_fn = lambda btn: None
 
+        # If the wallet can populate the inputs with more info, do it now.
+        # As a result, e.g. we might learn an imported address tx is segwit,
+        # or that a beyond-gap-limit address is is_mine.
+        # note: this might fetch prev txs over the network.
+        tx.add_info_from_wallet(self.wallet)
+
     def on_open(self):
         self.update()
 
@@ -129,6 +151,7 @@ class TxDialog(Factory.Popup):
         self.description = tx_details.label
         self.can_broadcast = tx_details.can_broadcast
         self.can_rbf = tx_details.can_bump
+        self.can_dscancel = tx_details.can_dscancel
         self.tx_hash = tx_details.txid or ''
         if tx_mined_status.timestamp:
             self.date_label = _('Date')
@@ -140,6 +163,7 @@ class TxDialog(Factory.Popup):
             self.date_label = ''
             self.date_str = ''
 
+        self.can_sign = self.wallet.can_sign(self.tx)
         if amount is None:
             self.amount_str = _("Transaction unrelated to your wallet")
         elif amount > 0:
@@ -148,10 +172,23 @@ class TxDialog(Factory.Popup):
         else:
             self.is_mine = True
             self.amount_str = format_amount(-amount)
-        self.fee_str = format_amount(fee) if fee is not None else _('unknown')
-        self.can_sign = self.wallet.can_sign(self.tx)
-        self.ids.output_list.update(self.tx.get_outputs_for_UI())
-        self.is_local_tx = tx_mined_status.height == TX_HEIGHT_LOCAL
+        risk_of_burning_coins = (isinstance(self.tx, PartialTransaction)
+                                 and self.can_sign
+                                 and fee is not None
+                                 and bool(self.wallet.get_warning_for_risk_of_burning_coins_as_fees(self.tx)))
+        if fee is not None and not risk_of_burning_coins:
+            self.fee_str = format_amount(fee)
+            fee_per_kb = fee / self.tx.estimated_size() * 1000
+            self.feerate_str = self.app.format_fee_rate(fee_per_kb)
+        else:
+            self.fee_str = _('unknown')
+            self.feerate_str = _('unknown')
+        self.ids.output_list.update(self.tx.outputs())
+
+        for dict_entry in self.ids.output_list.data:
+            dict_entry['color'], dict_entry['background_color'] = address_colors(self.wallet, dict_entry['address'])
+
+        self.can_remove_tx = tx_details.can_remove
         self.update_action_button()
 
     def update_action_button(self):
@@ -160,7 +197,8 @@ class TxDialog(Factory.Popup):
             ActionButtonOption(text=_('Sign'), func=lambda btn: self.do_sign(), enabled=self.can_sign),
             ActionButtonOption(text=_('Broadcast'), func=lambda btn: self.do_broadcast(), enabled=self.can_broadcast),
             ActionButtonOption(text=_('Bump fee'), func=lambda btn: self.do_rbf(), enabled=self.can_rbf),
-            ActionButtonOption(text=_('Remove'), func=lambda btn: self.remove_local_tx(), enabled=self.is_local_tx),
+            ActionButtonOption(text=_('Cancel (double-spend)'), func=lambda btn: self.do_dscancel(), enabled=self.can_dscancel),
+            ActionButtonOption(text=_('Remove'), func=lambda btn: self.remove_local_tx(), enabled=self.can_remove_tx),
         )
         num_options = sum(map(lambda o: bool(o.enabled), options))
         # if no options available, hide button
@@ -184,7 +222,7 @@ class TxDialog(Factory.Popup):
             self._action_button_fn = dropdown.open
             for option in options:
                 if option.enabled:
-                    btn = Button(text=option.text, size_hint_y=None, height=48)
+                    btn = Button(text=option.text, size_hint_y=None, height='48dp')
                     btn.bind(on_release=option.func)
                     dropdown.add_widget(btn)
 
@@ -202,16 +240,13 @@ class TxDialog(Factory.Popup):
         d = BumpFeeDialog(self.app, fee, size, self._do_rbf)
         d.open()
 
-    def _do_rbf(self, old_fee, new_fee, is_final):
-        if new_fee is None:
-            return
-        delta = new_fee - old_fee
-        if delta < 0:
-            self.app.show_error("fee too low")
+    def _do_rbf(self, new_fee_rate, is_final):
+        if new_fee_rate is None:
             return
         try:
-            new_tx = self.wallet.bump_fee(self.tx, delta)
-        except BaseException as e:
+            new_tx = self.wallet.bump_fee(tx=self.tx,
+                                          new_fee_rate=new_fee_rate)
+        except CannotBumpFee as e:
             self.app.show_error(str(e))
             return
         if is_final:
@@ -220,8 +255,31 @@ class TxDialog(Factory.Popup):
         self.update()
         self.do_sign()
 
+    def do_dscancel(self):
+        from .dscancel_dialog import DSCancelDialog
+        is_relevant, is_mine, v, fee = self.wallet.get_wallet_delta(self.tx)
+        if fee is None:
+            self.app.show_error(_('Cannot cancel transaction') + ': ' + _('unknown fee for original transaction'))
+            return
+        size = self.tx.estimated_size()
+        d = DSCancelDialog(self.app, fee, size, self._do_dscancel)
+        d.open()
+
+    def _do_dscancel(self, new_fee_rate):
+        if new_fee_rate is None:
+            return
+        try:
+            new_tx = self.wallet.dscancel(tx=self.tx,
+                                          new_fee_rate=new_fee_rate)
+        except CannotDoubleSpendTx as e:
+            self.app.show_error(str(e))
+            return
+        self.tx = new_tx
+        self.update()
+        self.do_sign()
+
     def do_sign(self):
-        self.app.protected(_("Enter your PIN code in order to sign this transaction"), self._do_sign, ())
+        self.app.protected(_("Sign this transaction?"), self._do_sign, ())
 
     def _do_sign(self, password):
         self.status_str = _('Signing') + '...'
@@ -238,11 +296,9 @@ class TxDialog(Factory.Popup):
         self.app.broadcast(self.tx)
 
     def show_qr(self):
-        from electrum.bitcoin import base_encode, bfh
-        raw_tx = str(self.tx)
-        text = bfh(raw_tx)
-        text = base_encode(text, base=43)
-        self.app.qr_dialog(_("Raw Transaction"), text, text_for_clipboard=raw_tx)
+        original_raw_tx = str(self.tx)
+        qr_data = self.tx.to_qr_data()
+        self.app.qr_dialog(_("Raw Transaction"), qr_data, text_for_clipboard=original_raw_tx)
 
     def remove_local_tx(self):
         txid = self.tx.txid()
@@ -257,8 +313,19 @@ class TxDialog(Factory.Popup):
             if b:
                 for tx in to_delete:
                     self.wallet.remove_transaction(tx)
-                self.wallet.storage.write()
+                self.wallet.save_db()
                 self.app._trigger_update_wallet()  # FIXME private...
                 self.dismiss()
         d = Question(question, on_prompt)
         d.open()
+
+    def label_dialog(self):
+        from .label_dialog import LabelDialog
+        key = self.tx.txid()
+        text = self.app.wallet.get_label_for_txid(key)
+        def callback(text):
+            self.app.wallet.set_label(key, text)
+            self.update()
+            self.app.history_screen.update()
+        d = LabelDialog(_('Enter Transaction Label'), text, callback)
+        d.open()
